import React, { useEffect, useMemo } from 'react';
import { SettingsContainer } from '../../components/config';
import { layerSettings, layerTypes } from './layers';
import { Input } from '../../..';
import { VisualizationSettingsPropTypes } from '../../common';
import { MapEdgeData, MapNodeData, MapNodeOrEdgeData, MapProps } from './mapvis';

export const MapSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<MapProps>) => {
  const DataLayerSettings = settings.layer && layerSettings?.[settings.layer];

  const spatialAttributes = useMemo(() => {
    return Object.fromEntries(
      graphMetadata.nodes.labels.map((node) => [
        node,
        Object.entries(graphMetadata.nodes.types[node].attributes)
          .filter(([, value]) => value.dimension === 'numerical')
          .map(([key]) => key),
      ]),
    );
  }, [graphMetadata]);

  useEffect(() => {
    const nodes = Object.fromEntries(
      graphMetadata.nodes.labels.map(
        (node) =>
          [
            node,
            {
              color: [252, 185, 0],
              hidden: false,
              fixed: true,
              min: 0,
              max: 10,
              radius: 1,
              lat: undefined,
              lon: undefined,
              collapsed: false,
              shape: 'circle',
              size: 10,
              ...(settings?.nodes && node in settings.nodes ? settings.nodes[node] : {}),
            },
          ] as [string, MapNodeData],
      ),
    );

    const edges = Object.fromEntries(
      graphMetadata.edges.labels.map(
        (edge) =>
          [
            edge,
            {
              color: [171, 184, 195],
              hidden: false,
              fixed: true,
              min: 0,
              max: 10,
              radius: 1,
              sizeAttribute: undefined,
              collapsed: false,
              size: 10,
              ...(settings?.edges && edge in settings.edges ? settings.edges[edge] : {}),
            },
          ] as [string, MapEdgeData],
      ),
    );

    updateSettings({ nodes: nodes, edges: edges });
  }, [graphMetadata]);

  useEffect(() => {
    // Autodetect a lat or lon attribute if not already set
    Object.keys(settings.nodes).forEach((node) => {
      if ((!settings.nodes[node].lat || !settings.nodes[node].lon) && node in spatialAttributes) {
        const lat = spatialAttributes[node].find((attr) => attr.includes('latitude'));
        const lon = spatialAttributes[node].find((attr) => attr.includes('longitude'));
        if (lat && lon) {
          updateSettings({ nodes: { ...settings.nodes, [node]: { ...settings.nodes[node], lat, lon } } });
        }
      }
    });
  }, [spatialAttributes, settings]);

  return (
    <SettingsContainer>
      <Input
        label="Data layer"
        type="dropdown"
        inline
        value={settings.layer}
        options={Object.keys(layerTypes)}
        onChange={(val) => updateSettings({ layer: val as string })}
      />
      {DataLayerSettings && !!spatialAttributes && (
        <DataLayerSettings
          settings={settings}
          graphMetadata={graphMetadata}
          updateSettings={updateSettings}
          spatialAttributes={spatialAttributes}
        />
      )}
    </SettingsContainer>
  );
};
