import React from 'react';
import { CompositeLayer, Layer } from 'deck.gl';
import { IconLayer, LineLayer, ScatterplotLayer, TextLayer } from '@deck.gl/layers';
import { CompositeLayerType, LayerProps } from '../../mapvis.types';
import { BrushingExtension, CollisionFilterExtension } from '@deck.gl/extensions';
import { Edge, Node } from '@graphpolaris/shared/lib/data-access';
import { createIcon } from './shapeFactory';

export class NodeLinkLayer extends CompositeLayer<CompositeLayerType> {
  static type = 'NodeLink';
  private _layers: Record<string, Layer> = {};

  constructor(props: LayerProps) {
    super(props);
  }

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  renderLayers() {
    const { data, settings, getNodeLocation, selected, setLayerIds, graphMetadata } = this.props;

    const brushingExtension = new BrushingExtension();
    const collisionFilter = new CollisionFilterExtension();

    graphMetadata.edges.labels.forEach((label: string) => {
      const layerId = `${label}-edges-line`;
      const edgeData =
        selected.length > 0 ? data.edges.filter((edge: Edge) => selected.includes(edge.from) || selected.includes(edge.to)) : data.edges;

      this._layers[layerId] = new LineLayer<Edge>({
        id: layerId,
        data: edgeData,
        visible: !settings.edges[label].hidden,
        pickable: true,
        getWidth: settings.edges[label].width,
        getSourcePosition: (d) => getNodeLocation(d.from),
        getTargetPosition: (d) => getNodeLocation(d.to),
        getColor: (d) => settings.edges[d.label].color,
        extensions: [brushingExtension],
      });
    });
    graphMetadata.nodes.labels.forEach((label: string) => {
      const layerId = `${label}-nodes-scatterplot`;

      this._layers[layerId] = new IconLayer<Node>({
        id: layerId,
        visible: !settings.nodes[label].hidden,
        data: data.nodes.filter((node: Node) => node.label === label),
        pickable: true,
        getColor: (d) => [200, 140, 0],
        getSize: (d) => settings.nodes[label].size,
        getPosition: (d) => getNodeLocation(d._id),
        getIcon: (d: any) => {
          return {
            url: createIcon(settings.nodes[label].shape, settings.nodes[label].color),
            width: 24,
            height: 24,
          };
        },
      });
    });

    const textLayerId = 'label-target';

    this._layers[textLayerId] = new TextLayer({
      id: textLayerId,
      data: data.nodes,
      getPosition: (d: any) => getNodeLocation(d._id),
      getText: (d: any) => d.id,
      getSize: 15,
      visible: true,
      getAlignmentBaseline: 'top',
      background: true,
      getPixelOffset: [10, 10],
      extensions: [collisionFilter],
      collisionEnabled: true,
      getCollisionPriority: (d: any) => d.id,
      collisionTestProps: { sizeScale: 10 },
      getRadius: 10,
      radiusUnits: 'pixels',
      collisionGroup: 'text',
    });

    return Object.values(this._layers);
  }
}

NodeLinkLayer.layerName = 'NodeLink';
