import React, { useEffect, useMemo, useCallback, useState, useRef } from 'react';
import DeckGL from '@deck.gl/react';
import { CompositeLayer, FlyToInterpolator, Position, WebMercatorViewport } from '@deck.gl/core';
import { SelectionLayer } from '@deck.gl-community/editable-layers';
import { CompositeLayerType, Coordinate, Layer } from './mapvis.types';
import { VISComponentType, VisualizationPropTypes } from '../../common';
import { layerTypes, createBaseMap, MapLayerSettingsPropTypes } from './layers';
import { MapSettings } from './MapSettings';
import { Node } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { SearchBar } from './SearchBar';
import { Icon } from '@graphpolaris/shared/lib/components';

export type MapNodeOrEdgeData = MapNodeData | MapEdgeData;

export type MapNodeData = {
  color: [number, number, number];
  hidden: boolean;
  fixed: boolean;
  min: number;
  max: number;
  radius: number;
  collapsed: boolean;
  lat?: string;
  lon?: string;
  shape: 'circle' | 'square' | 'triangle' | 'diamond' | 'location' | 'star';
  size: number;
  sizeAttribute?: string;
};

export type MapEdgeData = {
  color: [number, number, number];
  hidden: boolean;
  fixed: boolean;
  min: number;
  max: number;
  radius: number;
  collapsed: boolean;
  size: number;
  width: number;
  sizeAttribute?: string;
};

export type MapProps = {
  layer: string;
  enableBrushing: boolean;
  nodes: Record<string, MapNodeData>;
  edges: Record<string, MapEdgeData>;
};

const settings: MapProps = { layer: 'nodelink', enableBrushing: false, nodes: {}, edges: {} };

const INITIAL_VIEW_STATE = {
  latitude: 52.1006,
  longitude: 5.6464,
  zoom: 6,
  bearing: 0,
  pitch: 0,
};

const FLY_SPEED = 1000;

export const MapVis = (props: VisualizationPropTypes<MapProps>) => {
  const baseLayer = useRef(createBaseMap());
  const [viewport, setViewport] = useState<Record<string, any>>(INITIAL_VIEW_STATE);
  const [hoverObject, setHoverObject] = useState<Node | null>(null);
  const [selected, setSelected] = useState<any[]>([]);
  const [selectingRectangle, setSelectingRectangle] = useState<boolean>(false);
  const [layerIds, setLayerIds] = useState<string[]>([]);
  const [isSearching, setIsSearching] = useState<boolean>(false);

  const getFittedViewport = useCallback(
    (minLat: number, maxLat: number, minLon: number, maxLon: number) => {
      const viewportWebMercator = new WebMercatorViewport(viewport).fitBounds(
        [
          [minLon, minLat],
          [maxLon, maxLat],
        ],
        { padding: 20 },
      );
      const { zoom, longitude, latitude } = viewportWebMercator;
      return { zoom, longitude, latitude };
    },
    [viewport],
  );

  const flyToBoundingBox = useCallback(
    (minLat: number, maxLat: number, minLon: number, maxLon: number, options = {}) => {
      const fittedViewport = getFittedViewport(minLat, maxLat, minLon, maxLon);
      setViewport((prevViewport) => ({
        ...prevViewport,
        ...options,
        ...fittedViewport,
        transitionDuration: FLY_SPEED,
        transitionInterpolator: new FlyToInterpolator(),
      }));
    },
    [getFittedViewport],
  );

  const [dataLayer, setDataLayer] = useState<{ component: CompositeLayer<CompositeLayerType>; id: string } | null>(null);

  useEffect(() => {
    if (!props.settings.layer) {
      setDataLayer(null);
      return;
    }

    const layer = {
      component: props.settings.layer ? layerTypes?.[props.settings.layer] : layerTypes.nodelink,
      settings: props.settings,
      id: props.settings.layer,
    };

    const coordinateLookup: { [id: string]: Position } = props.data.nodes.reduce(
      (acc, node) => {
        const latitude = props.settings.nodes?.[node.label]?.lat
          ? (node?.attributes?.[props.settings.nodes[node.label].lat as any] as string)
          : undefined;
        const longitude = props.settings.nodes?.[node.label]?.lon
          ? (node?.attributes?.[props.settings.nodes[node.label].lon as any] as string)
          : undefined;

        if (latitude !== undefined && longitude !== undefined) {
          acc[node._id] = [parseFloat(longitude), parseFloat(latitude)];
        }

        return acc;
      },
      {} as { [id: string]: Position },
    );

    const layerProps: CompositeLayerType = {
      ...props,
      selected: selected,
      hoverObject: hoverObject,
      getNodeLocation: (d: string) => coordinateLookup[d],
      flyToBoundingBox: flyToBoundingBox,
      setLayerIds: (val: string[]) => setLayerIds(val),
    };

    if (dataLayer && dataLayer.id === layer.id) {
      // dataLayer.updateState;
      setDataLayer({ component: dataLayer.component.clone(layerProps), id: props.settings.layer });
    } else {
      // @ts-ignore
      setDataLayer({ component: new layer.component(layerProps), id: props.settings.layer });
    }
  }, [props.settings.layer, props.data, selected, hoverObject, props.settings]);

  const selectionLayer = useMemo(
    () =>
      selectingRectangle &&
      new (SelectionLayer as any)({
        id: 'selection',
        selectionType: 'rectangle',
        onSelect: ({ pickingInfos }: { pickingInfos: any[] }) => {
          if (pickingInfos.length > 0) {
            const nodes = [];
            const edges = [];

            for (const selectedItem of pickingInfos) {
              const { object } = selectedItem;
              if (object._id) {
                if (object.from & object.to) {
                  edges.push(object);
                } else {
                  nodes.push(object);
                }
              }
            }
            setSelected(nodes.map((node) => node._id));
            props.handleSelect({ nodes, edges });
          } else {
            props.handleSelect();
          }
          setSelectingRectangle(false);
        },
        layerIds: layerIds,
        getTentativeFillColor: () => [22, 37, 67, 100],
      }),
    [selectingRectangle],
  );

  return (
    <div className="w-full h-full flex-grow relative overflow-hidden">
      <div className="absolute left-0 top-0 z-50 m-1">
        <div className="cursor-pointer p-1 bg-white shadow-md rounded mb-1" onClick={() => setSelectingRectangle(true)}>
          <Icon component="icon-[ic--baseline-highlight-alt]" />
        </div>
        <div className="cursor-pointer p-1 bg-white shadow-md rounded" onClick={() => setIsSearching(!isSearching)}>
          <Icon component="icon-[ic--outline-search]" />
        </div>
      </div>
      {isSearching && (
        <SearchBar
          onSearch={(boundingBox: [number, number, number, number]) => {
            flyToBoundingBox(...boundingBox);
            setIsSearching(false);
          }}
        />
      )}
      <DeckGL
        layers={[baseLayer.current, dataLayer?.component, selectionLayer]}
        controller={true}
        initialViewState={viewport}
        onViewStateChange={({ viewState }) => setViewport(viewState)}
        onClick={({ object }) => {
          if (props.data) {
            if (!object) {
              props.handleSelect();
              setSelected([]);
              return;
            }
            if (object.hasOwnProperty('attributes') && object.hasOwnProperty('id') && object.hasOwnProperty('label')) {
              props.handleSelect({ nodes: [object] });
              setSelected([object.id]);
            }
            if (object.type === 'Feature') {
              const ids = object.properties.nodes;
              if (ids.length > 0) {
                const nodes = props.data.nodes.filter((node) => ids.includes((node as unknown as { id: string }).id));
                props.handleSelect({ nodes: [...nodes] });
              } else {
                props.handleSelect();
                setSelected([]);
                return;
              }
            }
          }
        }}
        onHover={({ object }) => {
          setHoverObject(object !== undefined ? object : null);
        }}
      />
      <div className="absolute right-0 top-0 p-1 z-50 bg-white bg-opacity-75 text-xs">
        {'© '}
        <a className="underline" href="http://www.openstreetmap.org/copyright" target="_blank" rel="noopener noreferrer">
          OpenStreetMap
        </a>
      </div>
    </div>
  );
};

export const MapComponent: VISComponentType<MapProps> = {
  displayName: 'MapVis',
  description: 'Geographical Features',
  component: MapVis,
  settingsComponent: MapSettings,
  settings: settings,
};

export default MapComponent;
